function dy = deriv(y, h, order, method, err)
%DERIV	Differentiates a matrix of data numerically
%
%   DERIV(Y) calculates the central differences of each vector
%   of matrix Y.
%
%   DERIV(Y,H) calculated the first-order derivative of Y by
%   central finite differences using H as the independent
%   variable interval.
%
%   DY = DERIV(Y,H,ORDER,METHOD,ERR) returns the derivative
%   of columns of the matrix Y where
%   H      is the independent variable interval
%   ORDER  is the order of differentiation (up to 4th order)
%   METHOD is the finite difference method used
%     Use METHOD = -1 for backward finite difference
%     Use METHOD =  0 for central finite difference
%     Use METHOD =  1 for forward finite difference
%   ERR    is the order of error of calculation. ERR may be
%     1 or 2 for backward and forward finite difference
%     and 2 or 4 for central finite difference.
%
%   See also FDER, DIFF

% (c) N. Mostoufi & A. Constantinides
% January 1, 1999

% Initialization
if nargin == 1 | isempty(h)
   h = 1;
end

if nargin < 3 | isempty(order)
   order = 1;
end

if nargin < 4 | isempty(method)
   method = 0;
   err = 2;
end

if nargin == 4
   if method == 0
      err = 2;
   else
      err = 1;
   end
end
if abs(method) == 1 & (err < 1 | err > 2)
   err = 1;
   warning(' Order of truncation error is set to 1.')
end
if method ==0 & ~(err ~= 2 | err ~= 4)
   err = 2;
   warning(' Order of truncation error is set to 2.')
end

[r , c] = size(y);
if r == 1	% If y is a row vector
   y = y';	% Make it a column vector
   r = c;
   c = 1;
end
n = r;	% Number of points
dy = zeros(r , c);

% Differentiation
switch method
case -1	% Backward finite differences
   switch err
   case 1	% O(h)
      switch order
      case 1	% 1st order derivative
         dy(2:n , :) = (y(2:n , :) - y(1:n-1 , :))/h;
         dy(1 , :) = (y(2 , :) - y(1 , :))/h;
      case 2	% 2nd order derivative
         dy(3:n , :) = (y(3:n , :) - 2*y(2:n-1 , :) + y(1:n-2 , :))/h^2;
         dy(1:2 , :) = (y(3:4 , :) - 2*y(2:3 , :) + y(1:2 , :))/h^2;
      case 3	% 3rd order derivative
         dy(4:n , :) = (y(4:n , :) - 3*y(3:n-1 , :) + 3*y(2:n-2 , :) ...
            - y(1:n-3 , :))/h^3;
         dy(1:3 , :) = (y(4:6 , :) - 3*y(3:5 , :) + 3*y(2:4 , :) - y(1:3 , :))/h^3;
      case 4	% 4th order derivative
         dy(5:n , :) = (y(5:n , :) - 4*y(4:n-1 , :) + 6*y(3:n-2 , :) ...
            - 4*y(2:n-3 , :) + y(1:n-4 , :))/h^4;
         dy(1:4 , :) = (y(5:8 , :) - 4*y(4:7 , :) + 6*y(3:6 , :) - 4*y(2:5 , :) ...
            + y(1:4 , :))/h^4;
      end
   case 2	% O(h^2)
      switch order
      case 1	% 1st order derivative
         dy(3:n , :) = (3*y(3:n , :) - 4*y(2:n-1 , :) + y(1:n-2 , :))/(2*h);
         dy(1:2 , :) = (-y(3:4 , :) + 4*y(2:3 , :) - 3*y(1:2 , :))/(2*h);
      case 2	% 2nd order derivative
         dy(4:n , :) = (2*y(4:n , :) - 5*y(3:n-1 , :) + 4*y(2:n-2 , :) ...
            - y(1:n-3 , :))/h^2;
         dy(1:3 , :) = (-y(4:6 , :) + 4*y(3:5 , :) - 5*y(2:4 , :) + 2*y(1:3 , :))/h^2;
      case 3	% 3rd order derivative
         dy(5:n , :) = (5*y(5:n , :) - 18*y(4:n-1 , :) + 24*y(3:n-2 , :) ...
            - 14*y(2:n-3 , :) + 3*y(1:n-4 , :))/(2*h^3);
         dy(1:4 , :) = (-3*y(5:8 , :) + 14*y(4:7 , :) - 24*y(3:6 , :) ...
            + 18*y(2:5 , :) - 5*y(1:4 , :))/(2*h^3);
      case 4	% 4th order derivative
         dy(6:n , :) = (3*y(6:n , :) - 14*y(5:n-1 , :) + 26*y(4:n-2 , :) ...
            - 24*y(3:n-3 , :) + 11*y(2:n-4 , :) - 2*y(1:n-5 , :))/h^4;
         dy(1:5 , :) = (-2*y(6:10 , :) + 11*y(5:9 , :) - 24*y(4:8 , :) ...
            + 26*y(3:7 , :) - 14*y(2:6 , :) + 3*y(1:5 , :))/h^4;
      end
   end
case 0	% Central finite differences
   switch err
   case 2	% O(h^2)
      switch order
      case 1	% 1st order derivative
         dy(1 , :) = (-y(3 , :) + 4*y(2 , :) - 3*y(1 , :))/(2*h);
         dy(2:n-1 , :) = (y(3:n , :) - y(1:n-2 , :))/(2*h);
         dy(n , :) = (3*y(n , :) - 4*y(n-1 , :) + y(n-2 , :))/(2*h);
      case 2	% 2nd order derivative
         dy(1 , :) = (-y(4 , :) + 4*y(3 , :) - 5*y(2 , :) + 2*y(1 , :))/h^2;
         dy(2:n-1 , :) = (y(3:n , :) - 2*y(2:n-1 , :) + y(1:n-2 , :))/h^2;
         dy(n , :) = (2*y(n , :) - 5*y(n-1 , :) + 4*y(n-2 , :) - y(n-3 , :))/h^2;
      case 3	% 3rd order derivative
         dy(1:2 , :) = (-3*y(5:6 , :) + 14*y(4:5 , :) - 24*y(3:4 , :) ...
            + 18*y(2:3 , :) - 5*y(1:2 , :))/(2*h^3);
         dy(3:n-2 , :) = (y(5:n , :) - 2*y(4:n-1 , :) + 2*y(2:n-3 , :) ...
            - y(1:n-4 , :))/(2*h^3);
         dy(n-1:n , :) = (5*y(n-1:n , :) - 18*y(n-2:n-1 , :) ...
            + 24*y(n-3:n-2 , :) - 14*y(n-4:n-3 , :) + 3*y(n-5:n-4 , :))/(2*h^3);
      case 4	% 4th order derivative
         dy(1:2 , :) = (-2*y(6:7 , :) + 11*y(5:6 , :) - 24*y(4:5 , :) ...
            + 26*y(3:4 , :) - 14*y(2:3 , :) + 3*y(1:2 , :))/h^4;
         dy(3:n-2 , :) = (y(5:n , :) - 4*y(4:n-1 , :) + 6*y(3:n-2 , :) ...
            - 4*y(2:n-3 , :) + y(1:n-4 , :))/h^4;
         dy(n-1:n , :) = (3*y(n-1:n , :) - 14*y(n-2:n-1 , :) ...
            + 26*y(n-3:n-2 , :) - 24*y(n-4:n-3 , :) + 11*y(n-5:n-4 , :) ...
            - 2*y(n-6:n-5 , :))/h^4;
      end
   case 4	% O(h^4)
      switch order
      case 1
         dy(1:2 , :) = (-y(3:4 , :) + 4*y(2:3 , :) - 3*y(1:2 , :))/(2*h);
         dy(3:n-2 , :) = (-y(5:n , :) + 8*y(4:n-1 , :) - 8*y(2:n-3 , :) ...
            + y(1:n-4 , :))/(12*h);
         dy(n-1:n , :) = (3*y(n-1:n , :) - 4*y(n-2:n-1 , :) + y(n-3:n-2 , :)) ...
            /(2*h);
      case 2
         dy(1:2 , :) = (-y(4:5 , :) + 4*y(3:4 , :) - 5*y(2:3 , :) + 2*y(1:2 , :))/h^2;
         dy(3:n-2 , :) = (-y(5:n , :) + 16*y(4:n-1 , :) - 30*y(3:n-2 , :) ...
            + 16*y(2:n-3 , :) - y(1:n-4 , :))/(12*h^2);
         dy(n-1:n , :) = (2*y(n-1:n , :) - 5*y(n-2:n-1 , :) + 4*y(n-3:n-2 , :) ...
            - y(n-4:n-3 , :))/h^2;
      case 3
         dy(1:3 , :) = (-3*y(5:7 , :) + 14*y(4:6 , :) - 24*y(3:5 , :) + 18*y(2:4 , :) ...
            - 5*y(1:3 , :))/(2*h^3);
         dy(4:n-3 , :) = (-y(7:n , :) + 8*y(6:n-1 , :) - 13*y(5:n-2 , :) ...
            + 13*y(3:n-4 , :) - 8*y(2:n-5 , :) + y(1:n-6 , :))/(8*h^3);
         dy(n-2:n , :) = (5*y(n-2:n , :) - 18*y(n-3:n-1 , :) ...
            + 24*y(n-4:n-2 , :) - 14*y(n-5:n-3 , :) + 3*y(n-6:n-4 , :))/ (2*h^3);
      case 4
         dy(1:3 , :) = (-2*y(6:8 , :) + 11*y(5:7 , :) - 24*y(4:6 , :) + 26*y(3:5 , :) ...
            - 14*y(2:4 , :) + 3*y(1:3 , :))/h^4;
         dy(4:n-3 , :) = (-y(7:n , :) + 12*y(6:n-1 , :) - 39*y(5:n-2 , :) ...
            + 56*y(4:n-3 , :) - 39*y(3:n-4 , :) + 12*y(2:n-5 , :) - y(1:n-6 , :)) ...
            /(6*h^4);
         dy(n-2:n , :) = (3*y(n-2:n , :) - 14*y(n-3:n-1 , :) + 26*y(n-4:n-2 , :) ...
            - 24*y(n-5:n-3 , :) + 11*y(n-6:n-4 , :) - 2*y(n-7:n-5 , :))/h^4;
      end
   end
case 1	% Forward finite differences
   switch err
   case 1	% O(h)
      switch order
      case 1	% 1st order derivative
         dy(1:n-1 , :) = (y(2:n , :) - y(1:n-1 , :))/h;
         dy(n , :) = (y(n , :) - y(n-1 , :))/h;
      case 2	% 2nd order derivative
         dy(1:n-2 , :) = (y(3:n , :) - 2*y(2:n-1 , :) + y(1:n-2 , :))/h^2;
         dy(n-1:n , :) = (y(n-1:n , :) - 2*y(n-2:n-1 , :) + y(n-3:n-2 , :))/h^2;
      case 3	% 3rd order derivative
         dy(1:n-3 , :) = (y(4:n , :) - 3*y(3:n-1 , :) + 3*y(2:n-2 , :) ...
            - y(1:n-3 , :))/h^3;
         dy(n-2:n , :) = (y(n-2:n , :) - 3*y(n-3:n-1 , :) + 3*y(n-4:n-2 , :) ...
            - y(n-5:n-3 , :))/h^3;
      case 4	% 4th order derivative
         dy(1:n-4 , :) = (y(5:n , :) - 4*y(4:n-1 , :) + 6*y(3:n-2 , :) ...
            - 4*y(2:n-3 , :) + y(1:n-4 , :))/h^4;
         dy(n-3:n , :) = (y(n-3:n , :) - 4*y(n-4:n-1 , :) + 6*y(n-5:n-2 , :) ...
            - 4*y(n-6:n-3 , :) + y(n-7:n-4 , :))/h^4;
      end
   case 2	% O(h^2)
      switch order
      case 1	% 1st order derivative
         dy(1:n-2 , :) = (-y(3:n , :) + 4*y(2:n-1 , :) - 3*y(1:n-2 , :))/(2*h);
         dy(n-1:n , :) = (3*y(n-1:n , :) - 4*y(n-2:n-1 , :) + y(n-3:n-2 , :)) ...
            /(2*h);
      case 2	% 2nd order derivative
         dy(1:n-3 , :) = (-y(4:n , :) + 4*y(3:n-1 , :) - 5*y(2:n-2 , :) ...
            + 2*y(1:n-3 , :))/h^2;
         dy(n-2:n , :) = (2*y(n-2:n , :) - 5*y(n-3:n-1 , :) + 4*y(n-4:n-2 , :) ...
            - y(n-5:n-3 , :))/h^2;
      case 3	% 3rd order derivative
         dy(1:n-4 , :) = (-3*y(5:n , :) + 14*y(4:n-1 , :) - 24*y(3:n-2 , :) ...
            + 18*y(2:n-3 , :) - 5*y(1:n-4 , :))/(2*h^3);
         dy(n-3:n , :) = (5*y(n-3:n , :) - 18*y(n-4:n-1 , :) + 24*y(n-5:n-2 , :) ...
            - 14*y(n-6:n-3 , :) + 3*y(n-7:n-4 , :))/(2*h^3);
      case 4	% 4th order derivative
         dy(1:n-5 , :) = (-2*y(6:n , :) + 11*y(5:n-1 , :) - 24*y(4:n-2 , :) ...
            + 26*y(3:n-3 , :) - 14*y(2:n-4 , :) + 3*y(1:n-5 , :))/h^4;
         dy(n-4:n , :) = (3*y(n-4:n , :) - 14*y(n-5:n-1 , :) + 26*y(n-6:n-2 , :) ...
            - 24*y(n-7:n-3 , :) + 11*y(n-8:n-4 , :) - 2*y(n-9:n-5 , :))/h^4;
      end
   end
end
