function [x,y] = Adams(ODEfile,xi,xf,h,yi,varargin)
% ADAMS Solves a set of ordinary differential equations by the
%   Adams method.
%
%   [X,Y]=ADAMS('F',XI,XF,H,YI) solves a set of ordinary
%   differential equations by the Adams method, from XI to XF.
%   The equations are given in the M-file F.M.  H is the length 
%   of the interval and YI is the vector of initial values of 
%   the dependent variable at XI.
%
%   [X,Y]=ADAMS('F',XI,XF,H,YI,P1,P2,...) allows for additional
%   arguments which are passed to the function F(X,P1,P2,...).
%
%  See also ODE23, ODE45, ODE113, ODE15S, ODE23S, EULER, MEULER,
%           RK, ADAMSMOULTON

% (c) N. Mostoufi & A. Constantinides
% January 1, 1999

% Initialization
if isempty(h) | h == 0
   h = (xf - xi)/99;
end

yi = (yi(:).');	% Make sure it's a row vector

x = [xi:h:xf]';	% Vector of x values
if x(end) ~= xf
   x(end+1) = xf;
end
d = diff(x);		% Vector of x-increments

% Starting values
[a,b]=RK(ODEfile,x(1),x(3),h,yi,3,varargin{:});
y(:,1:3) = b;
for i = 1:3
   f(:,i) = feval(ODEfile,x(i),y(:,i),varargin{:});
end

% Solution
for i = 3:length(x)-1
   y(:,i+1) = y(:,i) + d(i)/12 * (23*f(:,i) - 16*f(:,i-1) + 5*f(:,i-2));
   f(:,i+1) = feval(ODEfile,x(i+1),y(:,i+1),varargin{:});
end
