function [x,y] = MEuler(ODEfile,xi,xf,h,yi,varargin)
% MEULER Solves a set of ordinary differential equations by
%   the modified Euler (predictor-corrector) method.
%
%   [X,Y]=MEULER('F',XI,XF,H,YI) solves a set of ordinary
%   differential equations by the modified Euler (the Euler
%   predictor-corrector) method, from XI to XF.
%   The equations are given in the M-file F.M.  H is the length of
%   interval and YI is the vector of initial values of the dependent
%   variable at XI.
%
%   [X,Y]=MEULER('F',XI,XF,H,YI,P1,P2,...) allows for additional
%   arguments which are passed to the function F(X,P1,P2,...).
%
%  See also ODE23, ODE45, ODE113, ODE15S, ODE23S, EULER, RK,
%           ADAMS, ADAMSMOULTON

% (c) N. Mostoufi & A. Constantinides
% January 1, 1999

% Initialization
if isempty(h) | h == 0
   h = (xf - xi)/99;
end

yi = (yi(:).')';	% Make sure it's a column vector

x = [xi:h:xf];		% Vector of x values
if x(end) ~= xf
   x(end+1) = xf;
end
d = diff(x);		% Vector of x-increments

y(:,1) = yi;		% Initial condition
% Solution
for i = 1:length(x)-1
   % Predictor
   y(:,i+1) = y(:,i) + d(i) * feval(ODEfile,x(i),y(:,i),varargin{:});
   % Corrector
   y(:,i+1) = y(:,i) + d(i) * feval(ODEfile,x(i+1),y(:,i+1),varargin{:});
end
